/**
 * Import layout widget controller
 *
 * @class
 * @since 3.0.0
 *
 * @param {MessageService}                        MessageService                           Service to show messages
 * @param {ZhLayoutPreferences}                   ZhLayoutPreferences                      Service to handle layout preferences methods
 * @param {ZhPreferences}                         ZhPreferences                            Service to handle preferences persistance
 * @param {ZhLayoutWidgetsBuilder}                ZhLayoutWidgetsBuilder Service to create widgets to handle layouts
 * @param {ZHPromise}                             ZHPromise                                Service to handle promises
 * @param {ScreenService}                         ScreenService                            Service to manipulate screen
 */
function ZhLayoutImportLayoutController(MessageService, ZhLayoutPreferences, ZhPreferences, ZhLayoutWidgetsBuilder, ZHPromise, ScreenService){

    var importLayoutWidgetPromise = null;

    /**
     * Retrieve widget to import layout
     *
     * @since 3.0.0
     *
     * @return {Promise<Widget>}
     */
    this.getImportLayoutWidget = function(){
        if(importLayoutWidgetPromise === null){
            importLayoutWidgetPromise = ZhLayoutWidgetsBuilder.factoryImportLayoutWidget();
        }

        return importLayoutWidgetPromise;
    };

    /**
     * Import layout
     *
     * @since 3.0.0
     *
     * @param  {Widget} importLayoutWidget Widget with the layout configuration
     *
     * @return {Promise}                      Promise resolved after the layout is imported
     */
    this.importPreference = function(importLayoutWidget){
        if(!importLayoutWidget.isValid()){
            return ZHPromise.reject({err: 'WIDGET_INVALID'});
        }

        var layoutFileData = importLayoutWidget.fields[0].value()[0].b64File;
        layoutFileData = layoutFileData.split('base64,')[1];
        layoutFileData = window.atob(layoutFileData);
        layoutFileData = decodeURIComponent(escape(layoutFileData));

        var layout = JSON.parse(layoutFileData);

        var configWidget = importLayoutWidget.configWidget;
        var listWidget = configWidget.listWidget;

        if(!ZhLayoutPreferences.validateLayoutImport(layout, configWidget)){
            return ZHPromise.reject({err: 'LAYOUT_FILE_INVALID'});
        }

        layout.__is_new = true;
        layout.__generate_id = false;
        layout.LAYOUT_CHANGES = JSON.stringify(layout.LAYOUT_CHANGES);

        return ZhPreferences.getUserID().then(function(userID){
            layout.USER_ID = userID;

            return ZhLayoutPreferences.saveLayouts([layout]).then(function(){
                ScreenService.closePopup();
                ZhLayoutPreferences.updateLayouts(listWidget.id, configWidget.dataSource);
                configWidget.refreshWidget();
            }).catch(function(error){
                MessageService.showMessageByCode('zh-layout-preferences#save_error', {
                    'ERROR': error.message
                });
            });
        });
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register('ZhLayoutImportLayoutController', ZhLayoutImportLayoutController);
});